/* -*- mode: Java; c-basic-offset: 4; indent-tabs-mode: nil; -*-  //------100-columns-wide------>|*/
/*
 * Copyright (c) 2002-2004 Extreme! Lab, Indiana University. All rights reserved.
 *
 * This software is open source. See the bottom of this file for the license.
 *
 * $Id: HttpClientConnectionManager.java,v 1.8 2006/11/20 03:24:48 aslom Exp $
 */

package xsul.http_client;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.Socket;
import java.net.SocketException;
import xsul.http_client.plain_impl.PlainClientSocketFactory;

/**
 * Manages connection.
 * <br />NOTE: this class is multi-thread safe!
 *
 * @version $Revision: 1.8 $
 * @author <a href="http://www.extreme.indiana.edu/~aslom/">Aleksander Slominski</a>
 */
public class HttpClientConnectionManager {
    protected static ClientSocketFactory DEFAULT_SOCKET_FACTORY = PlainClientSocketFactory.getDefault();

    protected ClientSocketFactory socketFactory = DEFAULT_SOCKET_FACTORY;

    protected HttpClientConnectionManager()
    {
    }

    protected HttpClientConnectionManager(ClientSocketFactory socketFactory)
    {
        this.socketFactory = socketFactory;
    }

    public static HttpClientConnectionManager newInstance() {
        return new HttpClientConnectionManager();
    }

    public static HttpClientConnectionManager newInstance(ClientSocketFactory socketFactory) {
        return new HttpClientConnectionManager(socketFactory);
    }


    /**
     * As defined in Socket a timeout of zero is interpreted as an infinite timeout
     */
    public HttpClientRequest connect(String host, int port, int timeout)
        throws HttpClientException
    {
        if(host == null) throw new IllegalArgumentException();

        //opening new connection
        Socket socket = socketFactory.connect(host, port, timeout);
        try {
            socket.setTcpNoDelay(true);
            socket.setSoLinger(false, 0);
        } catch (SocketException e) {
            throw new HttpClientException(
                "TCP no delay (disabling Nagle algorithm ) support is good for performance");
        }
        ClientSocketConnection conn = new ClientSocketConnection(host, port, timeout, socket);
        return new HttpClientRequest(conn, this);
    }

    /**
     * This method is called by HttpClientRequest/Response when HTTP request/response
     * is finished and it is safe to assume that thsi particular conenctions is finished
     */
    public void notifyConnectionForReuse(ClientSocketConnection conn) throws HttpClientException
    {
        // forceful closing of socket
        conn.close();
    }

    /**
     * This method should be overriden by subclasses to free any resources that were not released.
     * This simple class has no resources to reclaim - this manager (factory) is stateless.
     */
    public void shutdownAndReclaimResources() throws HttpClientException{
    }

}


/*
 * Indiana University Extreme! Lab Software License, Version 1.2
 *
 * Copyright (c) 2002-2004 The Trustees of Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 * 1) All redistributions of source code must retain the above
 *    copyright notice, the list of authors in the original source
 *    code, this list of conditions and the disclaimer listed in this
 *    license;
 *
 * 2) All redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the disclaimer
 *    listed in this license in the documentation and/or other
 *    materials provided with the distribution;
 *
 * 3) Any documentation included with all redistributions must include
 *    the following acknowledgement:
 *
 *      "This product includes software developed by the Indiana
 *      University Extreme! Lab.  For further information please visit
 *      http://www.extreme.indiana.edu/"
 *
 *    Alternatively, this acknowledgment may appear in the software
 *    itself, and wherever such third-party acknowledgments normally
 *    appear.
 *
 * 4) The name "Indiana University" or "Indiana University
 *    Extreme! Lab" shall not be used to endorse or promote
 *    products derived from this software without prior written
 *    permission from Indiana University.  For written permission,
 *    please contact http://www.extreme.indiana.edu/.
 *
 * 5) Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 * Indiana University provides no reassurances that the source code
 * provided does not infringe the patent or any other intellectual
 * property rights of any other entity.  Indiana University disclaims any
 * liability to any recipient for claims brought by any other entity
 * based on infringement of intellectual property rights or otherwise.
 *
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH
 * NO WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA
 * UNIVERSITY GIVES NO WARRANTIES AND MAKES NO REPRESENTATION THAT
 * SOFTWARE IS FREE OF INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR
 * OTHER PROPRIETARY RIGHTS.  INDIANA UNIVERSITY MAKES NO WARRANTIES THAT
 * SOFTWARE IS FREE FROM "BUGS", "VIRUSES", "TROJAN HORSES", "TRAP
 * DOORS", "WORMS", OR OTHER HARMFUL CODE.  LICENSEE ASSUMES THE ENTIRE
 * RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR ASSOCIATED MATERIALS,
 * AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION GENERATED USING
 * SOFTWARE.
 */



