/* -*- mode: Java; c-basic-offset: 4; indent-tabs-mode: nil; -*-  //------100-columns-wide------>|*/
/* Copyright (c) 2006 Extreme! Lab, Indiana University. All rights reserved.
 * This software is open source. See the bottom of this file for the license.
 * $Id: BaseXmlElementAdapter.java,v 1.9 2007/10/03 21:09:43 smarru Exp $ */
package xsul.lead;

import java.net.URI;
import org.xmlpull.v1.builder.XmlAttribute;
import org.xmlpull.v1.builder.XmlElement;
import org.xmlpull.v1.builder.XmlInfosetBuilder;
import org.xmlpull.v1.builder.XmlNamespace;
import org.xmlpull.v1.builder.adapter.XmlElementAdapter;
import xsul.MLogger;
import xsul.XmlConstants;
import xsul.ws_addressing.WsaEndpointReference;

/**
 * Base class with utility methods to manipulate XML.
 */
public abstract class BaseXmlElementAdapter extends XmlElementAdapter {
    private final static MLogger logger = MLogger.getLogger();
    private final static XmlInfosetBuilder builder = XmlConstants.BUILDER;

    public BaseXmlElementAdapter(XmlElement target) {
        super(target);
    }

    protected void setIntegerAttributeValue(String name, Integer value) {
        String v;
        if(value != null) {
            v = value.toString();
        } else {
            v = null;
        }
        setAttributeValue(name, v);
    }

    protected Integer getIntegerAttributeValue(String name) {
        String v = getAttributeValue(null, name);
        if(v == null) return null;
        return new Integer(v);
    }
    
    protected void setBooleanAttributeValue(String name, boolean bool) {
        String v = Boolean.toString(bool);
        setAttributeValue(name, v);
    }

    protected boolean getBooleanAttributeValue(String name) {
        String v = getAttributeValue(null, name);
        if(v == null) return true;
        return Boolean.parseBoolean(v);
    }

    protected void setUriAttributeValue(String name, URI uri) {
        String v;
        if(uri != null) {
            v = uri.toASCIIString();
        } else {
            v = null;
        }
        setAttributeValue(name, v);
    }

    protected URI getUriAttributeValue(String name) {
        String v = getAttributeValue(null, name);
        if(v == null) return null;
        return URI.create(v);
    }

    protected void setAttributeValue(String name, String value) {
        XmlAttribute a = attribute(name);
        if(a != null) removeAttribute(a);
        // if value == null then attribute is removed
        if(value != null) addAttribute(name, value);
    }

    protected String getAttributeValue(String name) {
        return getAttributeValue(null, name);
    }

//    protected void setStringSecondEl(XmlNamespace NS, String name, String value) {
//        XmlElement el = element(NS, name);
//        if(el == null) {
//            // make sure element are inserted after experiment Id
//            el = newElement(NS, name);
//            addChild(1, el);
//            el.setParent(this);
//        }
//        el.replaceChildrenWithText(value);
//    }

//    protected void setStringAppend(XmlNamespace NS, String name, String value) {
//        XmlElement el = element(NS, name, true);
//        el.replaceChildrenWithText(value);
//    }

    protected void setStringValue(XmlNamespace NS, String name, String value) {
        XmlElement el = element(NS, name, true);
        el.replaceChildrenWithText(value);
    }

    protected void setRequiredStringValue(XmlNamespace NS, String name, String value) {
        if(value == null) throw new IllegalArgumentException();
        setStringValue(NS, name, value);
    }

    protected String getString(XmlNamespace NS, String name) {
        XmlElement el = element(NS, name);
        if(el == null) {
            return null;
        }
        return el.requiredTextContent();
    }

    protected String requireString(XmlNamespace NS, String name) {
        String value = getString(NS, name);
        if(value == null) throw new IllegalStateException("could not find required element "+name);
        return value;
    }

    protected String lookupStringValue(XmlNamespace NS, String name) {
        XmlElement el = element(NS, name);
        if(el == null) {
            return null;
        }
        return el.requiredTextContent();
    }

    protected void setUriValue(XmlNamespace NS, String name, URI value) {
        String s = value.toASCIIString();
        setStringValue(NS, name, s);
    }

//    protected void setUriValueSecondEl(XmlNamespace NS, String name, URI value) {
//        String s = value.toASCIIString();
//        setStringSecondEl(NS, name, s);
//    }

    protected URI lookupUriValue(XmlNamespace NS, String name) {
        String svalue = lookupStringValue(NS, name);
        if(svalue == null) return null;
        URI uri = URI.create(svalue);
        return uri;
    }

    protected void setEpr(WsaEndpointReference epr, XmlNamespace ns, String name) {
        //epr.setNamespace(builder.newNamespace(XmlElement.NO_NAMESPACE)); //CHECKME
        epr.setNamespace(ns);
        epr.setName(name);
        XmlElement el = element(ns, name);
        if(el != null) {
            replaceChild(epr, el);
        } else {
            addChild(1, epr); //addChild(epr);
        }
    }

    protected  WsaEndpointReference lookupEpr(XmlNamespace ns, String name) {
        XmlElement el = element(ns, name);
        if(el == null) {
            return null;
        }
        if(el instanceof WsaEndpointReference) {
            return (WsaEndpointReference) el;
        } else {
            return new WsaEndpointReference(el);
        }
    }
}

/*
 * Indiana University Extreme! Lab Software License, Version 1.2
 *
 * Copyright (c) 2006 The Trustees of Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 * 1) All redistributions of source code must retain the above
 *    copyright notice, the list of authors in the original source
 *    code, this list of conditions and the disclaimer listed in this
 *    license;
 *
 * 2) All redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the disclaimer
 *    listed in this license in the documentation and/or other
 *    materials provided with the distribution;
 *
 * 3) Any documentation included with all redistributions must include
 *    the following acknowledgement:
 *
 *      "This product includes software developed by the Indiana
 *      University Extreme! Lab.  For further information please visit
 *      http://www.extreme.indiana.edu/"
 *
 *    Alternatively, this acknowledgment may appear in the software
 *    itself, and wherever such third-party acknowledgments normally
 *    appear.
 *
 * 4) The name "Indiana University" or "Indiana University
 *    Extreme! Lab" shall not be used to endorse or promote
 *    products derived from this software without prior written
 *    permission from Indiana University.  For written permission,
 *    please contact http://www.extreme.indiana.edu/.
 *
 * 5) Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 * Indiana University provides no reassurances that the source code
 * provided does not infringe the patent or any other intellectual
 * property rights of any other entity.  Indiana University disclaims any
 * liability to any recipient for claims brought by any other entity
 * based on infringement of intellectual property rights or otherwise.
 *
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH
 * NO WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA
 * UNIVERSITY GIVES NO WARRANTIES AND MAKES NO REPRESENTATION THAT
 * SOFTWARE IS FREE OF INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR
 * OTHER PROPRIETARY RIGHTS.  INDIANA UNIVERSITY MAKES NO WARRANTIES THAT
 * SOFTWARE IS FREE FROM "BUGS", "VIRUSES", "TROJAN HORSES", "TRAP
 * DOORS", "WORMS", OR OTHER HARMFUL CODE.  LICENSEE ASSUMES THE ENTIRE
 * RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR ASSOCIATED MATERIALS,
 * AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION GENERATED USING
 * SOFTWARE.
 */




