/* -*- mode: Java; c-basic-offset: 4; indent-tabs-mode: nil; -*-  //------100-columns-wide------>|*/
/* Copyright (c) 2002-2005 Extreme! Lab, Indiana University. All rights reserved.
 * This software is open source. See the bottom of this file for the license.
 * $Id: LeadContextHeader.java,v 1.34 2007/09/27 17:23:09 smarru Exp $ */
package xsul.lead;

import java.net.URI;
import java.util.Iterator;

import javax.xml.namespace.QName;

import org.xmlpull.v1.builder.XmlElement;
import org.xmlpull.v1.builder.XmlInfosetBuilder;
import org.xmlpull.v1.builder.XmlNamespace;

import xsul.MLogger;
import xsul.XmlConstants;
import xsul.ws_addressing.WsaEndpointReference;

/**
 * Represents SOAP Header with context information for LEAD workflows/services.
 */
public class LeadContextHeader extends BaseXmlElementAdapter {
    private final static MLogger logger = MLogger.getLogger();
    private final static XmlInfosetBuilder builder = XmlConstants.BUILDER;
    public static final String GFAC_NAMESPACE = "http://www.extreme.indiana.edu/namespaces/2004/01/gFac";

    public final static QName TYPE = new QName(
        "http://lead.extreme.indiana.edu/namespaces/2005/10/lead-context-header", "context");
    public final static XmlNamespace NS = builder.newNamespace("lh", TYPE.getNamespaceURI());

    public final static String EXPERIMENT_ID = "experiment-id";
    public final static String WORKFLOW_INSTANCE_ID = "workflow-instance-id";
    public final static String WORKFLOW_TEMPLATE_ID = "workflow-template-id";
    public final static String NODE_ID = "workflow-node-id";
    public final static String TIME_STEP = "workflow-time-step";
    public final static String SERVICE_INSTANCE_ID = "service-instance-id";
    public final static String SERVICE_REPLICA_ID = "service-replica-id";
    //public final static String PARENT_SERVICE_INSTANCE_ID = "parent-service-instance-id";
    //public final static String SERVICE_TYPE_ID = "service-type-id";
    public final static String GFAC_URL = "gfac-url";
    public final static String XREGISTRY_URL = "xregistry-url";
    public final static String DSC_URL = "dsc-url";
    public final static String MYLEAD_AGENT_URL = "mylead-agent-url";
    public final static String RESOURCE_CATALOG_URL = "resource-catalog-url";
    public final static String RESOURCE_BROKER_URL = "resource-broker-url";
    public final static String RESOURCE_SCHEDULER= "resource-scheduler";
    public final static String LEAD_RESOURCE_SCHEDULER_ENUM= "LEAD";
    public final static String VGRADS_RESOURCE_SCHEDULER_ENUM= "VGRADS";
    public final static String SPRUCE_RESOURCE_SCHEDULER_ENUM= "SPRUCE";
    //public final static String PROPERTIES_FILE_URL = "properties-file-url";
    public final static String EVENT_SINK_EPR = "event-sink-epr";
    public final static String ERROR_SINK_EPR = "error-sink-epr";
    public final static String USER_DN = "user-dn";
    public final static String URGENCY = "URGENCY";
    public final static String OUTPUT_DATA_DIRECTORY = "OUTPUT_DATA_DIRECTORY";
    public final static String OPENDAP_DIRECTORY = "OPENDAP_DIRECTORY"; // prefix
    public final static String FORCE_FILESTAGING_TO_WORKING_DIR = "ForceFileStagingToWorkDir";
    public final static String OUTPUT_DATA_FILES_SUFFIX = "OUTPUT_DATA_FILES_SUFFIX";
    public final static String EXECUTION_SESSION_ID = "execution-session-id";
    public final static String SCMS_URL = "scms-url";
    
    public LeadContextHeader(String experimentId, String userDn) {
        super(builder.newFragment(NS, TYPE.getLocalPart()));
        setExperimentId(experimentId); // NOTE: this make it first element
        setUserDn(userDn);
    }

    public LeadContextHeader(XmlElement target) {
        super(target);
    }


    public void setExperimentId(String experimentId) { setStringValue(NS, EXPERIMENT_ID, experimentId); }
    public String getExperimentId() { return getString(NS, EXPERIMENT_ID); }
    public void setWorkflowId(URI workflowId) { setWorkflowInstanceId(workflowId); }
    public URI getWorkflowId() { return getWorkflowInstanceId(); }
    public void setWorkflowInstanceId(URI workflowId) { setUriValue(NS, WORKFLOW_INSTANCE_ID, workflowId); }
    public URI getWorkflowInstanceId() { return lookupUriValue(NS, WORKFLOW_INSTANCE_ID); }
    public void setWorkflowTemplateId(URI workflowId) { setUriValue(NS, WORKFLOW_TEMPLATE_ID, workflowId); }
    public URI getWorkflowTemplateId() { return lookupUriValue(NS, WORKFLOW_TEMPLATE_ID); }
    public void setNodeId(String nodeId) { setStringValue(NS, NODE_ID, nodeId); }
    public String getNodeId() { return getString(NS, NODE_ID); }
    public void setTimeStep(String timeStep) { setStringValue(NS, TIME_STEP, timeStep); }
    public String getTimeStep() { return getString(NS, TIME_STEP); }
//    public void setServiceId(URI serviceId) { setServiceInstanceId(serviceId); }
//    public URI getServiceId() { return getServiceInstanceId(); }
    public void setServiceId(String serviceId) { setStringValue(NS, SERVICE_INSTANCE_ID, serviceId); }
    public String getServiceId() { return getString(NS, SERVICE_INSTANCE_ID); }
    public void setServiceInstanceId(URI serviceId) { setUriValue(NS, SERVICE_INSTANCE_ID, serviceId); }
    public URI getServiceInstanceId() { return lookupUriValue(NS, SERVICE_INSTANCE_ID); }
    public void setServiceReplicaId(URI serviceId) { setUriValue(NS, SERVICE_REPLICA_ID, serviceId); }
    public URI getServiceReplicaId() { return lookupUriValue(NS, SERVICE_REPLICA_ID); }
    public void setGfacUrl(URI url) { setUriValue(NS, GFAC_URL, url); }
    public URI getGfacUrl() { return lookupUriValue(NS, GFAC_URL); }
    public void setXRegistryUrl(URI url) { setUriValue(NS, XREGISTRY_URL, url); }
    public URI getXRegistryUrl() { return lookupUriValue(NS, XREGISTRY_URL); }
    public void setDscUrl(URI url) { setUriValue(NS, DSC_URL, url); }
    public URI getDscUrl() { return lookupUriValue(NS, DSC_URL); }
    public void setMyleadAgentUrl(URI url) { setUriValue(NS, MYLEAD_AGENT_URL, url); }
    public URI getMyleadAgentUrl() { return lookupUriValue(NS, MYLEAD_AGENT_URL); }
    //    public void setPropertiesFileUrl(String url) { setStringSecondEl(PROPERTIES_FILE_URL, url); }
    //    public String getPropertiesFileUrl() { return getString(PROPERTIES_FILE_URL); }
    public void setResourceCatalogUrl(URI value) { setUriValue(NS, RESOURCE_CATALOG_URL, value); }
    public URI getResourceCatalogUrl() { return lookupUriValue(NS, RESOURCE_CATALOG_URL); }
    public void setResourceBrokerUrl(URI value) { setUriValue(NS, RESOURCE_BROKER_URL, value); }
    public URI getResourceBrokerUrl() { return lookupUriValue(NS, RESOURCE_BROKER_URL); }
    public void setResourceScheduler(String value) { setStringValue(NS, RESOURCE_SCHEDULER, value); }
    public String getResourceScheduler() { return getString(NS, RESOURCE_SCHEDULER); }
    public void setUserDn(String userDn) { setStringValue(NS, USER_DN, userDn); }
    public String getUserDn() { return getString(NS, USER_DN); }
    
    public void setUrgency(String urgency) { setStringValue(NS, URGENCY, urgency); }
    public String getUrgency() { return lookupStringValue(NS, URGENCY); }
    
    public void setOutPutDataDir(String outPutDataDir) { setStringValue(NS, OUTPUT_DATA_DIRECTORY, outPutDataDir); }
    public String getOutPutDataDir() { return lookupStringValue(NS, OUTPUT_DATA_DIRECTORY); }
    
    public void setOpenDapPrfix(String opendapPrefix) { setStringValue(NS, OPENDAP_DIRECTORY, opendapPrefix); }
    public String getOpenDapPrfix() { return lookupStringValue(NS, OPENDAP_DIRECTORY); }
    
    
    public void setForceFileStagingToWorkDir(String forceFileStagingToWorkDir) { setStringValue(NS, FORCE_FILESTAGING_TO_WORKING_DIR, forceFileStagingToWorkDir); }
    public String getForceFileStagingToWorkDir() { return lookupStringValue(NS, FORCE_FILESTAGING_TO_WORKING_DIR); }
    
    public void setOutputDataFilesSuffix(String outputDataFilesSuffix) { setStringValue(NS, OUTPUT_DATA_FILES_SUFFIX, outputDataFilesSuffix); }
    public String getOutputDataFilesSuffix() { return lookupStringValue(NS, OUTPUT_DATA_FILES_SUFFIX); }

    public void setExecutionSessionId(String executionSessionId) { setStringValue(NS, EXECUTION_SESSION_ID, executionSessionId); }
    public String getExecutionSessionId() { return lookupStringValue(NS, EXECUTION_SESSION_ID); }
    
    public void setScmsUrl(URI value) { setUriValue(NS, SCMS_URL, value); }
    public URI getScmsUrl() { return lookupUriValue(NS, SCMS_URL); }
    
    public LeadResourceMappingList getResourceMappingList() {
        XmlElement el = element(LeadResourceMappingList.NS, LeadResourceMappingList.TYPE.getLocalPart());
        if(el == null) return null;
        return (LeadResourceMappingList) castOrWrap(el, LeadResourceMappingList.class);
    }
    public void setResourceMappingList(LeadResourceMappingList rml) {
        // only one resource mapping list allowed - deleted old before setting new
        XmlElement el = element(LeadResourceMappingList.NS, LeadResourceMappingList.TYPE.getLocalPart());
        if(el != null) removeChild(el);
        addElement(rml);
    }

    public LeadResourceMapping getResourceMapping() {
        XmlElement el = element(LeadResourceMapping.NS, LeadResourceMapping.TYPE.getLocalPart());
        if(el == null) return null;
        return (LeadResourceMapping) castOrWrap(el, LeadResourceMapping.class);
    }
    public void setResourceMapping(LeadResourceMapping rm) {
        // only one resource mapping list allowed - deleted old before setting new
        XmlElement el = element(LeadResourceMapping.NS, LeadResourceMapping.TYPE.getLocalPart());
        if(el != null) removeChild(el);
        addElement(rm);
    }

    public void setEventSink(WsaEndpointReference epr) {
        setEpr(epr, NS, EVENT_SINK_EPR);
    }

    public WsaEndpointReference getEventSink() {
        return lookupEpr(NS, EVENT_SINK_EPR);
    }

    public void setErrorSink(WsaEndpointReference epr) {
        setEpr(epr, NS, ERROR_SINK_EPR);
    }

    public WsaEndpointReference getErrorSink() {
        return lookupEpr(NS, ERROR_SINK_EPR);
    }
    
    public Iterator getGfacExtensions(){
    	org.xmlpull.v1.builder.Iterable extensions = elements(builder.newNamespace(GFAC_NAMESPACE), null);
    	if(extensions != null){
    		return extensions.iterator();
	      }
    	return null;
    }
}

/*
 * Indiana University Extreme! Lab Software License, Version 1.2
 *
 * Copyright (c) 2002-2005 The Trustees of Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 * 1) All redistributions of source code must retain the above
 *    copyright notice, the list of authors in the original source
 *    code, this list of conditions and the disclaimer listed in this
 *    license;
 *
 * 2) All redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the disclaimer
 *    listed in this license in the documentation and/or other
 *    materials provided with the distribution;
 *
 * 3) Any documentation included with all redistributions must include
 *    the following acknowledgement:
 *
 *      "This product includes software developed by the Indiana
 *      University Extreme! Lab.  For further information please visit
 *      http://www.extreme.indiana.edu/"
 *
 *    Alternatively, this acknowledgment may appear in the software
 *    itself, and wherever such third-party acknowledgments normally
 *    appear.
 *
 * 4) The name "Indiana University" or "Indiana University
 *    Extreme! Lab" shall not be used to endorse or promote
 *    products derived from this software without prior written
 *    permission from Indiana University.  For written permission,
 *    please contact http://www.extreme.indiana.edu/.
 *
 * 5) Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 * Indiana University provides no reassurances that the source code
 * provided does not infringe the patent or any other intellectual
 * property rights of any other entity.  Indiana University disclaims any
 * liability to any recipient for claims brought by any other entity
 * based on infringement of intellectual property rights or otherwise.
 *
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH
 * NO WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA
 * UNIVERSITY GIVES NO WARRANTIES AND MAKES NO REPRESENTATION THAT
 * SOFTWARE IS FREE OF INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR
 * OTHER PROPRIETARY RIGHTS.  INDIANA UNIVERSITY MAKES NO WARRANTIES THAT
 * SOFTWARE IS FREE FROM "BUGS", "VIRUSES", "TROJAN HORSES", "TRAP
 * DOORS", "WORMS", OR OTHER HARMFUL CODE.  LICENSEE ASSUMES THE ENTIRE
 * RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR ASSOCIATED MATERIALS,
 * AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION GENERATED USING
 * SOFTWARE.
 */

