/* -*- mode: Java; c-basic-offset: 4; indent-tabs-mode: nil; -*-  //------100-columns-wide------>|*/
/* Copyright (c) 2006 Extreme! Lab, Indiana University. All rights reserved.
 * This software is open source. See the bottom of this file for the licence.
 * $Id: LeadCrosscutParametersUtil.java,v 1.10 2008/06/25 17:53:34 smarru Exp $ */
package xsul.lead;

import java.util.Properties;
import javax.xml.namespace.QName;
//import org.apache.xmlbeans.XmlObject;
import org.xmlpull.v1.builder.XmlElement;
import org.xmlpull.v1.builder.XmlInfosetBuilder;
import org.xmlpull.v1.builder.XmlNamespace;

import sun.security.action.GetBooleanAction;
import xsul.MLogger;
import xsul.XmlConstants;
import xsul.XsulException;
import xsul.xbeans_util.XBeansUtil;

/**
 * Convenient set of constants and methods to access cross-cutting parameters
 * without XmlBeans
 */
public class LeadCrosscutParametersUtil extends XmlNameValueList {
    private final static MLogger logger = MLogger.getLogger();

    private final static XmlInfosetBuilder builder = XmlConstants.BUILDER;

    public final static String FORECAST_TIME = "fcst_time";

    public final static String START_DATE = "start_date";

    public final static String START_HOUR = "start_hour";

    public final static String CENTER_LONGITUDE = "ctrlon";

    public final static String CENTER_LATITUDE = "ctrlat";

    public final static String DX = "dx";

    public final static String DY = "dy";

    public final static String DZ = "dz";

    public final static String NX = "nx";

    public final static String NY = "ny";

    public final static String NZ = "nz";

    public final static String WEST_BC = "westbc";

    public final static String EAST_BC = "eastbc";

    public final static String SOUTH_BC = "southbc";

    public final static String NORTH_BC = "northbc";
    
    public final static String USE_LATEST = "use_latest";
    
    public final static String USER_MODIFIED_NAMELISTS = "user_modified_namelists";
    public final static String MP_PHYSICS = "mp_physics";     
    public final static String RA_LW_PHYSICS = "ra_lw_physics";        
    public final static String RA_SW_PHYSICS = "ra_sw_physics";    
    public final static String RADT  = "radt";                 
    public final static String SF_SFCLAY_PHYSICS = "sf_sfclay_physics";     
    public final static String SF_SURFACE_PHYSICS = "sf_surface_physics";  
    public final static String BL_PBL_PHYSICS = "bl_pbl_physics";        
    public final static String CU_PHYSICS  = "cu_physics";           
    public final static String CUDT = "cudt";                 
    public final static String IFSNOW  = "ifsnow";            
    public final static String W_DAMPING = "w_damping";           
    public final static String DIFF_OPT = "diff_opt";              
    public final static String KM_OPT  = "km_opt";             
    public final static String BASE_TEMP  = "base_temp";            
    public final static String KHDIF  = "khdif";             
    public final static String KVDIF= "kvdif";
    
    private final static QName COMPLEX_TYPE = new QName(
            "http://lead.extreme.indiana.edu/namespaces/2006/lead-crosscut-parameters/",
            "LeadCrosscutParameters");

    public final static XmlNamespace TYPE_NS = builder.newNamespace("lcp", COMPLEX_TYPE
            .getNamespaceURI());

//    public LeadCrosscutParametersUtil(XmlObject xmlObjectToWrap) {
//        this(XBeansUtil.xmlObjectToXmlElement(xmlObjectToWrap));
//        // validate();
//    }

    public LeadCrosscutParametersUtil(XmlElement elementToWrap) {
        super(elementToWrap);
        if (elementToWrap == null)
            throw new IllegalArgumentException("null");
    }

    public LeadCrosscutParametersUtil(QName topElQName) {
        super(topElQName);
        declareNamespace(TYPE_NS); // less of xmlns clutter in serialized XML
    }

    public LeadCrosscutParametersUtil() {
        this(XmlConstants.XBEANS_XML_FRAGMENT_QNAME);
    }

    public LeadCrosscutParametersUtil(Properties props) {
        this();
        setFromProperties(props);
    }

    public XmlNamespace parameterNs() {
        return TYPE_NS;
    }

    public Float getCenterLatitude() {
        return getFloat(CENTER_LATITUDE);
    }

    public void setCenterLatitude(float value) {
        setFloat(CENTER_LATITUDE, value);
    }

    public Float getCenterLongitude() {
        return getFloat(CENTER_LONGITUDE);
    }

    public void setCenterLongitude(float value) {
        setFloat(CENTER_LONGITUDE, value);
    }

    public Float getForecastTimeInHours() {
        return getFloat(FORECAST_TIME);
    }

    public void setForecastTimeInHours(float value) {
        setFloat(FORECAST_TIME, value);
    }

    public String getForecastStartDate() {
        return getString(START_DATE);
    }

    public void setForecastStartDate(String startDate) {
        setString(START_DATE, startDate);
    }

    public Integer getForecastStartHour() {
        return getInt(START_HOUR);
    }

    public void setForecastStartHour(int startHour) {
        setInt(START_HOUR, startHour);
    }

    public Float getWestBc() {
        return getFloat(WEST_BC);
    }

    public float requireWestBc() {
        return requireFloat(WEST_BC);
    }

    public void setWestBc(float value) {
        setFloat(WEST_BC, value);
    }

    public Float getEastBc() {
        return getFloat(EAST_BC);
    }

    public float requireEastBc() {
        return requireFloat(EAST_BC);
    }

    public void setEastBc(float value) {
        setFloat(EAST_BC, value);
    }

    public Float getSouthBc() {
        return getFloat(SOUTH_BC);
    }

    public float requireSouthBc() {
        return requireFloat(SOUTH_BC);
    }

    public void setSouthBc(float value) {
        setFloat(SOUTH_BC, value);
    }

    public Float getNorthBc() {
        return getFloat(NORTH_BC);
    }

    public float requireNorthBc() {
        return requireFloat(NORTH_BC);
    }

    public void setNorthBc(float value) {
        setFloat(NORTH_BC, value);
    }

    // public Float getDx() { return getFloat(DX); }
    // public void setDx(float value) { setString(DX, Float.toString(value)); }
    // public Float getDy() { return getFloat(DY); }
    // public void setDy(float value) { setString(DY, Float.toString(value)); }
    // public Float getDz() { return getFloat(DZ); }
    // public void setDz(float value) { setString(DZ, Float.toString(value)); }
    public Integer getDx() {
        return getInt(DX);
    }

    public void setDx(int value) {
        setInt(DX, value);
    }

    public Integer getDy() {
        return getInt(DY);
    }

    public void setDy(int value) {
        setInt(DY, value);
    }

    public Integer getDz() {
        return getInt(DZ);
    }

    public void setDz(int value) {
        setInt(DZ, value);
    }

    public Integer getNx() {
        return getInt(NX);
    }

    public void setNx(int value) {
        setInt(NX, value);
    }

    public Integer getNy() {
        return getInt(NY);
    }

    public void setNy(int value) {
        setInt(NY, value);
    }

    public Integer getNz() {
        return getInt(NZ);
    }

    public void setNz(int value) {
        setInt(NZ, value);
    }
    
    public Boolean getUseLatest() {
        return getBoolean(USE_LATEST);
    }
    
    public void setUseLatest(boolean value) {
        setBoolean(USE_LATEST, value);
    }

    public void setUserModifiedNamelists(boolean value) {
        setBoolean(USER_MODIFIED_NAMELISTS, value);
    }
    public Boolean getUserModifiedNamelists() {
        return getBoolean(USER_MODIFIED_NAMELISTS);
    }
    
    public void setMpPhysics(int value) {
        setInt(MP_PHYSICS, value);
    }
    public Integer getMpPhysics() {
        return getInt(MP_PHYSICS);
    }
    
    public void setRaLwPhysics(int value) {
        setInt(RA_LW_PHYSICS, value);
    }
    public Integer getRaLwPhysics() {
        return getInt(RA_LW_PHYSICS);
    }
    
    public void setRaSwPhysics(int value) {
        setInt(RA_SW_PHYSICS, value);
    }
    public Integer getRaSwPhysics() {
        return getInt(RA_SW_PHYSICS);
    }
    
    public void setRadt(float value) {
        setFloat(RADT, value);
    }
    public Float getRadt() {
        return getFloat(RADT);
    }
    
    public void setSfSfclayPhysics(int value) {
        setInt(SF_SFCLAY_PHYSICS, value);
    }
    public Integer getSfSfclayPhysics() {
        return getInt(SF_SFCLAY_PHYSICS);
    }
     
    public void setSfSurfacePhysics(int value) {
        setInt(SF_SURFACE_PHYSICS, value);
    }
    public Integer getSfSurfacePhysics() {
        return getInt(SF_SURFACE_PHYSICS);
    }
    
    public void setBlPblPhysics(int value) {
        setInt(BL_PBL_PHYSICS, value);
    }
    public Integer getBlPblPhysics() {
        return getInt(BL_PBL_PHYSICS);
    }
    
    public void setCuPhysics(int value) {
        setInt(CU_PHYSICS, value);
    }
    public Integer getCuPhysics() {
        return getInt(CU_PHYSICS);
    }
    
    public void setCudt(float value) {
        setFloat(CUDT, value);
    }
    public Float getCudt() {
        return getFloat(CUDT);
    }
    
    public void setIfSnow(int value) {
        setInt(IFSNOW, value);
    }
    public Integer getIfSnow() {
        return getInt(IFSNOW);
    }
    
    public void setWDamping(int value) {
        setInt(W_DAMPING, value);
    }
    public Integer getWDamping() {
        return getInt(W_DAMPING);
    }
    
    public void setDiffOpt(int value) {
        setInt(DIFF_OPT, value);
    }
    public Integer getDiffOpt() {
        return getInt(DIFF_OPT);
    }
    
    public void setKmOpt(int value) {
        setInt(KM_OPT, value);
    }
    public Integer getKmOpt() {
        return getInt(KM_OPT);
    }
    
    
    public void setBaseTemp(float value) {
        setFloat(BASE_TEMP, value);
    }
    public Float getBaseTemp() {
        return getFloat(BASE_TEMP);
    }
    
    public void setKhdif(float value) {
        setFloat(KHDIF, value);
    }
    public Float getKhdif() {
        return getFloat(KHDIF);
    }
    
    public void setKvdif(float value) {
        setFloat(KVDIF, value);
    }
    public Float getKvdif() {
        return getFloat(KVDIF);
    }
    
    // private void validate() throws XsulException{
    // String name = getName();
    // if(!name.equals(TYPE.getLocalPart())) {
    // throw new XsulException("expected element with "+TYPE.getLocalPart()+"
    // but got "+name);
    // }
    // XmlNamespace ns = getNamespace();
    // }
}

/*
 * Indiana University Extreme! Lab Software License, Version 1.2
 *
 * Copyright (c) 2006 The Trustees of Indiana University. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 *
 * 2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 *
 * 3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 *
 * "This product includes software developed by the Indiana University Extreme!
 * Lab. For further information please visit http://www.extreme.indiana.edu/"
 *
 * Alternatively, this acknowledgment may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 *
 * 4) The name "Indiana University" or "Indiana University Extreme! Lab" shall
 * not be used to endorse or promote products derived from this software without
 * prior written permission from Indiana University. For written permission,
 * please contact http://www.extreme.indiana.edu/.
 *
 * 5) Products derived from this software may not use "Indiana University" name
 * nor may "Indiana University" appear in their name, without prior written
 * permission of the Indiana University.
 *
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of any
 * other entity. Indiana University disclaims any liability to any recipient for
 * claims brought by any other entity based on infringement of intellectual
 * property rights or otherwise.
 *
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 * INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 * GENERATED USING SOFTWARE.
 */

