/* -*- mode: Java; c-basic-offset: 4; indent-tabs-mode: nil; -*-  //------100-columns-wide------>|*/
/* Copyright (c) 2006 Extreme! Lab, Indiana University. All rights reserved.
 * This software is open source. See the bottom of this file for the license.
 * $Id: LeadResourceMapping.java,v 1.12 2010/06/18 19:53:13 smarru Exp $ */
package xsul.lead;

import java.net.URI;
import javax.xml.namespace.QName;
import org.xmlpull.v1.builder.XmlElement;
import org.xmlpull.v1.builder.XmlInfosetBuilder;
import org.xmlpull.v1.builder.XmlNamespace;
import xsul.MLogger;
import xsul.XmlConstants;

/**
 * Represent one LEAD resource mapping as defined in lead-resource-mapping.xsd
 */
public class LeadResourceMapping extends BaseXmlElementAdapter {
    private final static MLogger logger = MLogger.getLogger();
    private final static XmlInfosetBuilder builder = XmlConstants.BUILDER;

    public final static QName TYPE = new QName(
            LeadResourceMappingList.TYPE.getNamespaceURI(), "resource-mapping");
    public final static XmlNamespace NS = LeadResourceMappingList.NS;

    public final static String WORKFLOW_NODE_ID = "workflow-node-id";
    public final static String SERVICE_ID = "service-id";
    public final static String HOST_NAME = "host-name";
    public final static String WSGRAM_PREFEREED = "wsgram-preferred";
    public final static String GATEKEEPER_EPR = "gatekeeper-epr";
    public final static String JOB_MANAGER = "job-manager";
    public final static String CPU_COUNT = "cpu-count";
    public final static String NODE_COUNT = "node-count";
    public final static String QUEUE_NAME = "queue-name";
	public final static String MAX_WALL_TIME = "max-wall-time";

    public LeadResourceMapping(String hostName) {
        super(builder.newFragment(NS, TYPE.getLocalPart()));
        setHostName(hostName);
    }

    public LeadResourceMapping(XmlElement target) {
        super(target);
    }

    public void setHostName(String hostName) { replaceChildrenWithText(hostName); }
    public String getHostName() { return requiredTextContent(); }
    public void setWorkflowNodeId(String workflowNodeId) { setAttributeValue(WORKFLOW_NODE_ID, workflowNodeId);  }
    public String getWorkflowNodeId() { return getAttributeValue(WORKFLOW_NODE_ID); }
    public void setServiceId(URI serviceId) { setUriAttributeValue(SERVICE_ID, serviceId);  }
    public URI getServiceId() { return getUriAttributeValue(SERVICE_ID); }
    public void setGatekeeperEPR(URI gatekeeperEPR) { setUriAttributeValue(GATEKEEPER_EPR, gatekeeperEPR);  }
    public URI getGatekeeperEPR() { return getUriAttributeValue(GATEKEEPER_EPR); }
    public void setJobManager(String jobManager) { setAttributeValue(JOB_MANAGER, jobManager);  }
    public String getJobManager() { return getAttributeValue(JOB_MANAGER); }
    public void setWSGramPreferred(boolean wsGramPreferred) { setBooleanAttributeValue(WSGRAM_PREFEREED, wsGramPreferred);  }
    public boolean getWSGramPreferred() { return getBooleanAttributeValue(WSGRAM_PREFEREED); }
    public void setNodeCount(Integer nodeCount) {
        setIntegerAttributeValue(NODE_COUNT, nodeCount);  }
    public Integer getNodeCount() { return getIntegerAttributeValue(NODE_COUNT); }
    public void setCPUCount(Integer nodeCount) {
        setIntegerAttributeValue(CPU_COUNT, nodeCount);  }
    public Integer getCPUCount() { return getIntegerAttributeValue(CPU_COUNT); }
    public void setQueueName(String queueName) { setAttributeValue(QUEUE_NAME, queueName);  }
    public String getQueueName() { return getAttributeValue(QUEUE_NAME); }
	public void setMaxWallTime(String maxWallTime) { setAttributeValue(MAX_WALL_TIME, maxWallTime);  }
    public String getMaxWallTime() { return getAttributeValue(MAX_WALL_TIME); }

}

/*
 * Indiana University Extreme! Lab Software License, Version 1.2
 *
 * Copyright (c) 2006 The Trustees of Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 * 1) All redistributions of source code must retain the above
 *    copyright notice, the list of authors in the original source
 *    code, this list of conditions and the disclaimer listed in this
 *    license;
 *
 * 2) All redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the disclaimer
 *    listed in this license in the documentation and/or other
 *    materials provided with the distribution;
 *
 * 3) Any documentation included with all redistributions must include
 *    the following acknowledgement:
 *
 *      "This product includes software developed by the Indiana
 *      University Extreme! Lab.  For further information please visit
 *      http://www.extreme.indiana.edu/"
 *
 *    Alternatively, this acknowledgment may appear in the software
 *    itself, and wherever such third-party acknowledgments normally
 *    appear.
 *
 * 4) The name "Indiana University" or "Indiana University
 *    Extreme! Lab" shall not be used to endorse or promote
 *    products derived from this software without prior written
 *    permission from Indiana University.  For written permission,
 *    please contact http://www.extreme.indiana.edu/.
 *
 * 5) Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 * Indiana University provides no reassurances that the source code
 * provided does not infringe the patent or any other intellectual
 * property rights of any other entity.  Indiana University disclaims any
 * liability to any recipient for claims brought by any other entity
 * based on infringement of intellectual property rights or otherwise.
 *
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH
 * NO WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA
 * UNIVERSITY GIVES NO WARRANTIES AND MAKES NO REPRESENTATION THAT
 * SOFTWARE IS FREE OF INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR
 * OTHER PROPRIETARY RIGHTS.  INDIANA UNIVERSITY MAKES NO WARRANTIES THAT
 * SOFTWARE IS FREE FROM "BUGS", "VIRUSES", "TROJAN HORSES", "TRAP
 * DOORS", "WORMS", OR OTHER HARMFUL CODE.  LICENSEE ASSUMES THE ENTIRE
 * RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR ASSOCIATED MATERIALS,
 * AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION GENERATED USING
 * SOFTWARE.
 */


