/* -*- mode: Java; c-basic-offset: 4; indent-tabs-mode: nil; -*-  //------100-columns-wide------>|*/
/* Copyright (c) 2006 Extreme! Lab, Indiana University. All rights reserved.
 * This software is open source. See the bottom of this file for the license.
 * $Id: LeadResourceMappingList.java,v 1.4 2006/11/20 03:24:50 aslom Exp $ */
package xsul.lead;

import java.util.Arrays;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import javax.xml.namespace.QName;
import org.xmlpull.v1.builder.XmlElement;
import org.xmlpull.v1.builder.XmlInfosetBuilder;
import org.xmlpull.v1.builder.XmlNamespace;
import org.xmlpull.v1.builder.adapter.XmlElementAdapter;
import xsul.MLogger;
import xsul.XmlConstants;

/**
 * Element that contains list of LEAD resource mappings.
 */
public class LeadResourceMappingList extends XmlElementAdapter {
    private final static MLogger logger = MLogger.getLogger();
    private final static XmlInfosetBuilder builder = XmlConstants.BUILDER;

    public final static QName TYPE = new QName(
        "http://lead.extreme.indiana.edu/namespaces/2006/lead-resource-mapping/", "resource-mappings");
    public final static XmlNamespace NS = builder.newNamespace("lrm", TYPE.getNamespaceURI());

    public LeadResourceMappingList(List<LeadResourceMapping> listOfResourceMappings) {
        super(builder.newFragment(NS, TYPE.getLocalPart()));
        setResourceMappings(listOfResourceMappings);
    }

    public LeadResourceMappingList(LeadResourceMapping... lrm) {
        super(builder.newFragment(NS, TYPE.getLocalPart()));
        List<LeadResourceMapping> listOfResourceMappings = Arrays.asList(lrm);
        setResourceMappings(listOfResourceMappings);
    }

    public LeadResourceMappingList(XmlElement target) {
        super(target);
    }

    public void addResourceMapping(LeadResourceMapping resourceMapping) {
        addElement(resourceMapping);
    }

    public void setResourceMappings(List<LeadResourceMapping> listOfResourceMappings) {
        removeAllChildren();
        if(listOfResourceMappings == null) return;
        for ( LeadResourceMapping resourceMapping : listOfResourceMappings ) {
            addElement(resourceMapping);
        }
    }

    public List<LeadResourceMapping> getResourceMappings() {
        List<LeadResourceMapping> listOfResourceMappings = new LinkedList<LeadResourceMapping> ();
        // LATER better in XPP5
        Iterator i = elements(LeadResourceMapping.NS, LeadResourceMapping.TYPE.getLocalPart()).iterator();
        while(i.hasNext() ) {
            XmlElement e = (XmlElement) i.next();
            // LATER no cast needed in XPP5
            LeadResourceMapping resMapping = (LeadResourceMapping) castOrWrap(e, LeadResourceMapping.class);
            listOfResourceMappings.add(resMapping);
        }
        return listOfResourceMappings;
    }

}

/*
 * Indiana University Extreme! Lab Software License, Version 1.2
 *
 * Copyright (c) 2006 The Trustees of Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 * 1) All redistributions of source code must retain the above
 *    copyright notice, the list of authors in the original source
 *    code, this list of conditions and the disclaimer listed in this
 *    license;
 *
 * 2) All redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the disclaimer
 *    listed in this license in the documentation and/or other
 *    materials provided with the distribution;
 *
 * 3) Any documentation included with all redistributions must include
 *    the following acknowledgement:
 *
 *      "This product includes software developed by the Indiana
 *      University Extreme! Lab.  For further information please visit
 *      http://www.extreme.indiana.edu/"
 *
 *    Alternatively, this acknowledgment may appear in the software
 *    itself, and wherever such third-party acknowledgments normally
 *    appear.
 *
 * 4) The name "Indiana University" or "Indiana University
 *    Extreme! Lab" shall not be used to endorse or promote
 *    products derived from this software without prior written
 *    permission from Indiana University.  For written permission,
 *    please contact http://www.extreme.indiana.edu/.
 *
 * 5) Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 * Indiana University provides no reassurances that the source code
 * provided does not infringe the patent or any other intellectual
 * property rights of any other entity.  Indiana University disclaims any
 * liability to any recipient for claims brought by any other entity
 * based on infringement of intellectual property rights or otherwise.
 *
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH
 * NO WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA
 * UNIVERSITY GIVES NO WARRANTIES AND MAKES NO REPRESENTATION THAT
 * SOFTWARE IS FREE OF INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR
 * OTHER PROPRIETARY RIGHTS.  INDIANA UNIVERSITY MAKES NO WARRANTIES THAT
 * SOFTWARE IS FREE FROM "BUGS", "VIRUSES", "TROJAN HORSES", "TRAP
 * DOORS", "WORMS", OR OTHER HARMFUL CODE.  LICENSEE ASSUMES THE ENTIRE
 * RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR ASSOCIATED MATERIALS,
 * AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION GENERATED USING
 * SOFTWARE.
 */





