/* -*- mode: Java; c-basic-offset: 4; indent-tabs-mode: nil; -*-  //------100-columns-wide------>|*/
/*
 * Copyright (c) 2005 Extreme! Lab, Indiana University. All rights reserved.
 *
 * This software is open source. See the bottom of this file for the license.
 *
 * $Id: XsulMonitoringStats.java,v 1.10 2006/11/20 03:24:51 aslom Exp $
 */
package xsul.monitoring;

import java.net.URI;
import org.xmlpull.v1.builder.XmlElement;
import org.xmlpull.v1.builder.XmlInfosetBuilder;
import org.xmlpull.v1.builder.XmlNamespace;
import org.xmlpull.v1.builder.adapter.XmlElementAdapter;
import xsul.MLogger;
import xsul.XmlConstants;
import xsul.XsulVersion;

/**
 *
 */
public class XsulMonitoringStats extends XmlElementAdapter {
    private final static MLogger logger = MLogger.getLogger();
    private final static XmlInfosetBuilder builder = XmlConstants.BUILDER;
    public final static XmlNamespace MONITORING_NS = builder.newNamespace(
        "m", "http://www.extreme.indiana.edu/xgws/xsul/monitoring/2006/");
    public final static String PING_REQUEST = "Ping";
    public final static String PING_RESPONSE = "PingResponse";
    
    public final static String SERVICE_STATS = "service-stats";
    public final static String SERVICE_NAME = "service-name";
    public final static String SERVICE_DESCRIPTION = "service-description";
    public final static String SERVICE_WSDL = "service-wsdl";
    public final static String SERVICE_WSDL_NAME = "name";
    public final static String TOOLKIT_NAME = "toolkit-name";
    public final static String STARTTIME_SECONDS = "starttime-seconds";
    public final static String XML_REQUESTS_COUNT = "xml-requests-count";
    //public final static String STARTTIME_MILISECONDS = "start-time-miliseconds-since-unix-epoch";
    public final static String XML_REQUESTS_PER_SECOND = "xml-requests-per-second";
    public final static String CONNECTIONS_COUNT = "connections-count";
    //public final static String SERVER_VERSION = "server-version";
    
    private static String defaultToolkitName = XsulVersion.getUserAgent();
    private String serviceName;
    private long startTimeMs;
    private long requestXmlMsgsCount;
    private URI[] serviceWsdls;
    private String[] serviceWsdlNames;
    
    protected XsulMonitoringStats(String serviceName,
                                  String serviceDescription,
                                  String[] serviceWsdlNames,
                                  URI[] serviceWsdls,
                                  long startTimeInMs,
                                  long requestXmlMsgCount,
                                  long numberOfConnections) {
        super(builder.newFragment(MONITORING_NS, SERVICE_STATS));
        this.serviceName = serviceName;
        this.serviceWsdls = serviceWsdls;
        this.serviceWsdlNames = serviceWsdlNames;
        this.startTimeMs = startTimeInMs;
        this.requestXmlMsgsCount = requestXmlMsgCount;
        {
            XmlElement serviceNameEl = addElement(MONITORING_NS, SERVICE_NAME);
            serviceNameEl.addChild(serviceName);
        }
        if(serviceDescription != null) {
            XmlElement serviceDescEl = addElement(MONITORING_NS, SERVICE_DESCRIPTION);
            serviceDescEl.addChild(serviceDescription);
        }
        if(serviceWsdls != null) {
            for (int i = 0; i < serviceWsdls.length; i++)
            {
                XmlElement serviceWsdlEl = addElement(MONITORING_NS, SERVICE_WSDL);
                serviceWsdlEl.addChild(serviceWsdls[i].toASCIIString());
                if(serviceWsdlNames != null) {
                    serviceWsdlEl.addAttribute(null, SERVICE_WSDL_NAME, serviceWsdlNames[i]);
                }
            }
        }
        {
            XmlElement toolkitNameEl = addElement(MONITORING_NS, TOOLKIT_NAME);
            toolkitNameEl.addChild(defaultToolkitName);
        }
        {
            XmlElement starttimeEl = addElement(MONITORING_NS, STARTTIME_SECONDS);
            starttimeEl.addChild(""+(long)(startTimeInMs/1000.0));
        }
        {
            XmlElement numberOfConnectionsEl = addElement(MONITORING_NS, CONNECTIONS_COUNT);
            numberOfConnectionsEl.addChild(""+numberOfConnections);
        }
        {
            XmlElement requestsXmlCountEl = addElement(MONITORING_NS, XML_REQUESTS_COUNT);
            requestsXmlCountEl.addChild(""+requestsXmlCountEl);
        }
    }
    
    public void setServiceWsdlNames(String[] serviceWsdlNames) {
        this.serviceWsdlNames = serviceWsdlNames;
    }
    
    public String[] getServiceWsdlNames() {
        return serviceWsdlNames;
    }
    
    
//    public void setServiceWsdls(URI[] serviceWsdls) {
//      this.serviceWsdls = serviceWsdls;
//    }
    
    public URI[] getServiceWsdls() {
        return serviceWsdls;
    }
    
    /**
     * @deprecated
     */
    public String getServerName() {
        return getServiceName();
    }
    
    public String getServiceName() {
        return serviceName;
    }
    
    public long getStarTimeInMs() {
        return startTimeMs;
    }
    
    public long getRequestXmlMsgsCount() {
        return requestXmlMsgsCount;
    }
    
    public long getConnectionsCount() {
        String s = requiredElement(null, CONNECTIONS_COUNT).requiredTextContent();
        return Long.parseLong(s);
    }
    
    public static void setDefaultToolkitName(String defaultToolkitName) {
        XsulMonitoringStats.defaultToolkitName = defaultToolkitName;
    }
    
    public static String getDefaultToolkitName() {
        return defaultToolkitName;
    }
    
}

/*
 * Indiana University Extreme! Lab Software License, Version 1.2
 *
 * Copyright (c) 2002-2004 The Trustees of Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 * 1) All redistributions of source code must retain the above
 *    copyright notice, the list of authors in the original source
 *    code, this list of conditions and the disclaimer listed in this
 *    license;
 *
 * 2) All redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the disclaimer
 *    listed in this license in the documentation and/or other
 *    materials provided with the distribution;
 *
 * 3) Any documentation included with all redistributions must include
 *    the following acknowledgement:
 *
 *      "This product includes software developed by the Indiana
 *      University Extreme! Lab.  For further information please visit
 *      http://www.extreme.indiana.edu/"
 *
 *    Alternatively, this acknowledgment may appear in the software
 *    itself, and wherever such third-party acknowledgments normally
 *    appear.
 *
 * 4) The name "Indiana University" or "Indiana University
 *    Extreme! Lab" shall not be used to endorse or promote
 *    products derived from this software without prior written
 *    permission from Indiana University.  For written permission,
 *    please contact http://www.extreme.indiana.edu/.
 *
 * 5) Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 * Indiana University provides no reassurances that the source code
 * provided does not infringe the patent or any other intellectual
 * property rights of any other entity.  Indiana University disclaims any
 * liability to any recipient for claims brought by any other entity
 * based on infringement of intellectual property rights or otherwise.
 *
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH
 * NO WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA
 * UNIVERSITY GIVES NO WARRANTIES AND MAKES NO REPRESENTATION THAT
 * SOFTWARE IS FREE OF INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR
 * OTHER PROPRIETARY RIGHTS.  INDIANA UNIVERSITY MAKES NO WARRANTIES THAT
 * SOFTWARE IS FREE FROM "BUGS", "VIRUSES", "TROJAN HORSES", "TRAP
 * DOORS", "WORMS", OR OTHER HARMFUL CODE.  LICENSEE ASSUMES THE ENTIRE
 * RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR ASSOCIATED MATERIALS,
 * AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION GENERATED USING
 * SOFTWARE.
 */



