/* -*- mode: Java; c-basic-offset: 4; indent-tabs-mode: nil; -*-  //------100-columns-wide------>|*/
/*
 * Copyright (c) 2002-2004 Extreme! Lab, Indiana University. All rights reserved.
 *
 * This software is open source. See the bottom of this file for the license.
 *
 * $Id: Provider.java,v 1.11 2006/12/09 20:05:33 aslom Exp $
 */

package xsul.wsif_xsul_soap_http;

import org.xmlpull.v1.builder.XmlElement;
import org.xmlpull.v1.builder.XmlInfosetBuilder;
import org.xmlpull.v1.builder.XmlNamespace;
import xsul.MLogger;
import xsul.XmlConstants;
import xsul.type_handler.TypeHandlerRegistry;
import xsul.util.XsulUtil;
import xsul.wsdl.WsdlBinding;
import xsul.wsdl.WsdlDefinitions;
import xsul.wsdl.WsdlPort;
import xsul.wsdl.WsdlUtil;
import xsul.wsif.WSIFException;
import xsul.wsif.WSIFPort;
import xsul.wsif.spi.WSIFProvider;
import xsul.wsif_xsul_soap_http.XsulSoapPort;


/**
 * XSUL Soap/HTTP provider for XWSIF
 *
 * @version $Revision: 1.11 $
 * @author <a href="http://www.extreme.indiana.edu/~aslom/">Aleksander Slominski</a>
 */
public class Provider implements WSIFProvider {
    private final static MLogger logger = MLogger.getLogger();
    private final static XmlInfosetBuilder builder = XmlConstants.BUILDER;
    private final static XmlNamespace wsdl = WsdlDefinitions.TYPE;
    public final static XmlNamespace WSDL_SOAP_NS = WsdlUtil.WSDL_SOAP_NS;
    public final static XmlNamespace WSDL_SOAP12_NS = WsdlUtil.WSDL_SOAP12_NS;

    public WSIFPort createDynamicWSIFPort(WsdlPort port,
                                          TypeHandlerRegistry typeMap)
        throws WSIFException
    {
        XmlElement soapAddr = port.element(WSDL_SOAP_NS, "address");
        if(soapAddr == null) {
            soapAddr = port.element(WSDL_SOAP12_NS, "address");
            if(soapAddr == null) {
                //return null;
                throw new WSIFException("could not find soap:address in "+errorContext(port));
            }
        }
        String location = soapAddr.getAttributeValue(null, "location");
        //if(! (location.startsWith("http://") || location.startsWith("https://"))) { //
        checkLocation(location, port);
        // get wsdl:binding
        WsdlBinding binding = port.lookupBinding();
        if(binding == null) {
            throw new WSIFException("no binding"+errorContext(port));
        }
        // check soap:binding is for SOAP ...
        XmlElement soapBinding  = binding.element(soapAddr.getNamespace(), "binding");
        if(soapBinding == null) { // no soap:binding?
            throw new WSIFException("could not find soap:binding in "+XsulUtil.safeXmlToString(binding)+errorContext(port));
        }
        //check binding style and transport
        String bindingTransport = soapBinding.getAttributeValue(null, "transport");
        if(! "http://schemas.xmlsoap.org/soap/http".equals(bindingTransport)) {
            throw new WSIFException("unsupported transport "+bindingTransport+" in "+XsulUtil.safeXmlToString(soapBinding)+errorContext(port));
        }
        assert bindingTransport != null;

        // extract binding style from soapBinding
        String bindingStyle = soapBinding.getAttributeValue(null, "style");
        if (bindingStyle == null) {
            bindingStyle = "document"; // default
        }
        if(! ("document".equals(bindingStyle)
                  || "rpc".equals(bindingStyle))) {
            throw new WSIFException("unsupported binding style "+bindingStyle+XsulUtil.safeXmlToString(soapBinding)+errorContext(port));
        }
        assert bindingStyle != null;

        // creat XsulSoapPort
        return newPort(port, soapAddr, location, binding, soapBinding, bindingStyle, typeMap);
    }

    protected void checkLocation(String location, WsdlPort port) throws WSIFException {
        if(! location.startsWith("http://")) {//
            throw new WSIFException("only HTTP is supported but got "+location+errorContext(port));
        }
    }

    protected XsulSoapPort newPort(WsdlPort port, XmlElement soapAddr,
                                   String location,
                                   WsdlBinding binding,
                                   XmlElement soapBinding,
                                   String bindingStyle,
                                   TypeHandlerRegistry typeMap) {
        return new XsulSoapPort(port, soapAddr, location, binding, soapBinding, bindingStyle, typeMap);
    }

    protected String errorContext(WsdlPort port) {
        return " (context: WSDL port="+ XsulUtil.safeXmlToString(port) +")";
    }



}


/*
 * Indiana University Extreme! Lab Software License, Version 1.2
 *
 * Copyright (c) 2002-2004 The Trustees of Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 * 1) All redistributions of source code must retain the above
 *    copyright notice, the list of authors in the original source
 *    code, this list of conditions and the disclaimer listed in this
 *    license;
 *
 * 2) All redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the disclaimer
 *    listed in this license in the documentation and/or other
 *    materials provided with the distribution;
 *
 * 3) Any documentation included with all redistributions must include
 *    the following acknowledgement:
 *
 *      "This product includes software developed by the Indiana
 *      University Extreme! Lab.  For further information please visit
 *      http://www.extreme.indiana.edu/"
 *
 *    Alternatively, this acknowledgment may appear in the software
 *    itself, and wherever such third-party acknowledgments normally
 *    appear.
 *
 * 4) The name "Indiana University" or "Indiana University
 *    Extreme! Lab" shall not be used to endorse or promote
 *    products derived from this software without prior written
 *    permission from Indiana University.  For written permission,
 *    please contact http://www.extreme.indiana.edu/.
 *
 * 5) Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 * Indiana University provides no reassurances that the source code
 * provided does not infringe the patent or any other intellectual
 * property rights of any other entity.  Indiana University disclaims any
 * liability to any recipient for claims brought by any other entity
 * based on infringement of intellectual property rights or otherwise.
 *
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH
 * NO WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA
 * UNIVERSITY GIVES NO WARRANTIES AND MAKES NO REPRESENTATION THAT
 * SOFTWARE IS FREE OF INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR
 * OTHER PROPRIETARY RIGHTS.  INDIANA UNIVERSITY MAKES NO WARRANTIES THAT
 * SOFTWARE IS FREE FROM "BUGS", "VIRUSES", "TROJAN HORSES", "TRAP
 * DOORS", "WORMS", OR OTHER HARMFUL CODE.  LICENSEE ASSUMES THE ENTIRE
 * RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR ASSOCIATED MATERIALS,
 * AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION GENERATED USING
 * SOFTWARE.
 */


