/* -*- mode: Java; c-basic-offset: 4; indent-tabs-mode: nil; -*-  //------100-columns-wide------>|*/
/*
 * Copyright (c) 2002-2004 Extreme! Lab, Indiana University. All rights reserved.
 *
 * This software is open source. See the bottom of this file for the license.
 *
 * $Id: XsulSoapPort.java,v 1.7 2006/12/19 00:01:27 aslom Exp $
 */

package xsul.wsif_xsul_soap_http;

import org.xmlpull.v1.builder.XmlElement;
import org.xmlpull.v1.builder.XmlNamespace;
import xsul.invoker.soap.SoapDynamicInfosetInvoker;
import xsul.invoker.soap_over_http.SoapHttpDynamicInfosetInvoker;
import xsul.soap12_util.Soap12Util;
import xsul.type_handler.TypeHandlerRegistry;
import xsul.wsdl.WsdlBinding;
import xsul.wsdl.WsdlBindingOperation;
import xsul.wsdl.WsdlPort;
import xsul.wsif.WSIFException;
import xsul.wsif.WSIFOperation;
import xsul.wsif.WSIFPort;


/**
 * XSUL Soap/HTTP communication endpoint.
 *
 * @version $Revision: 1.7 $
 * @author <a href="http://www.extreme.indiana.edu/~aslom/">Aleksander Slominski</a>
 */
public class XsulSoapPort implements WSIFPort {

    private WsdlPort port;
    private WsdlBinding binding;
    private TypeHandlerRegistry typeMap;
    private SoapDynamicInfosetInvoker invoker;
    private XmlNamespace soapNs;

    private String bindingStyle;

    private String location;

    XsulSoapPort(WsdlPort port,
                 XmlElement soapAddr,
                 String location,
                 WsdlBinding binding,
                 XmlElement soapBinding,
                 String bindingStyle,
                 TypeHandlerRegistry typeMap)
    {
        this.port = port;
        this.binding = binding;
        this.bindingStyle = bindingStyle;
        this.typeMap = typeMap;
        this.location=location;
        binding = port.lookupBinding();
        // check port has soap:address
        invoker = new SoapHttpDynamicInfosetInvoker();
        setInvoker(invoker);
        soapNs = soapAddr.getNamespace();
    }

//    public void setLocation(String location) {
//      this.location = location;
//    }

    public String getLocation() {
        return location;
    }

    public SoapDynamicInfosetInvoker getInvoker() {
        return invoker;
    }

    public void setInvoker(SoapDynamicInfosetInvoker invoker) {
        this.invoker = invoker;
        invoker.setLocation(location);
        if(Provider.WSDL_SOAP12_NS.equals(soapNs)) {
            invoker.setSoapFragrance(Soap12Util.getInstance());
        }
    }

    public WsdlPort getWsdlServicePort() throws WSIFException
    {
        return port;
    }

    public WSIFOperation createOperation(String operationName) throws WSIFException {
        WsdlBindingOperation bindingOp = binding.getOperation(operationName);
        if(bindingOp == null) {
            throw new WSIFException("binding "+binding.getBindingName()+" has no operation '"+operationName+"'");
        }
        return new XsulSoapOperation(this, bindingStyle, bindingOp, soapNs);
    }

    public void close() throws WSIFException {
    }

}


/*
 * Indiana University Extreme! Lab Software License, Version 1.2
 *
 * Copyright (c) 2002-2004 The Trustees of Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 * 1) All redistributions of source code must retain the above
 *    copyright notice, the list of authors in the original source
 *    code, this list of conditions and the disclaimer listed in this
 *    license;
 *
 * 2) All redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the disclaimer
 *    listed in this license in the documentation and/or other
 *    materials provided with the distribution;
 *
 * 3) Any documentation included with all redistributions must include
 *    the following acknowledgement:
 *
 *      "This product includes software developed by the Indiana
 *      University Extreme! Lab.  For further information please visit
 *      http://www.extreme.indiana.edu/"
 *
 *    Alternatively, this acknowledgment may appear in the software
 *    itself, and wherever such third-party acknowledgments normally
 *    appear.
 *
 * 4) The name "Indiana University" or "Indiana University
 *    Extreme! Lab" shall not be used to endorse or promote
 *    products derived from this software without prior written
 *    permission from Indiana University.  For written permission,
 *    please contact http://www.extreme.indiana.edu/.
 *
 * 5) Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 * Indiana University provides no reassurances that the source code
 * provided does not infringe the patent or any other intellectual
 * property rights of any other entity.  Indiana University disclaims any
 * liability to any recipient for claims brought by any other entity
 * based on infringement of intellectual property rights or otherwise.
 *
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH
 * NO WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA
 * UNIVERSITY GIVES NO WARRANTIES AND MAKES NO REPRESENTATION THAT
 * SOFTWARE IS FREE OF INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR
 * OTHER PROPRIETARY RIGHTS.  INDIANA UNIVERSITY MAKES NO WARRANTIES THAT
 * SOFTWARE IS FREE FROM "BUGS", "VIRUSES", "TROJAN HORSES", "TRAP
 * DOORS", "WORMS", OR OTHER HARMFUL CODE.  LICENSEE ASSUMES THE ENTIRE
 * RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR ASSOCIATED MATERIALS,
 * AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION GENERATED USING
 * SOFTWARE.
 */

