/* -*- mode: Java; c-basic-offset: 4; indent-tabs-mode: nil; -*-  //------100-columns-wide------>|*/
/*
 * Copyright (c) 2004 Extreme! Lab, Indiana University. All rights reserved.
 *
 * This software is open source. See the bottom of this file for the license.
 *
 * $Id: WSIFAsyncWsaResponsesCorrelatorBase.java,v 1.2 2006/11/20 03:25:01 aslom Exp $
 */
package xsul.xwsif_runtime_async;

import java.net.URI;
import java.util.HashMap;
import java.util.Map;
import org.xmlpull.v1.builder.XmlDocument;
import org.xmlpull.v1.builder.adapter.XmlElementAdapter;
import xsul.processor.DynamicInfosetProcessorException;
import xsul.util.FastUUIDGen;
import xsul.ws_addressing.WsaEndpointReference;
import xsul.ws_addressing.WsaMessageInformationHeaders;

/**
 * Simple base class to facilate implementing correlators.
 */
public abstract class WSIFAsyncWsaResponsesCorrelatorBase implements WSIFAsyncResponsesCorrelator {
    private Map messageId2Callback = new HashMap();
    private WsaEndpointReference replyToEpr;
    private URI replyToUri;
    
    protected WSIFAsyncWsaResponsesCorrelatorBase() {}
    
    protected void setReplyTo(WsaEndpointReference replyToEpr) {
        this.replyToEpr = replyToEpr;
    }
    
    protected void setReplyTo(URI replyToUri) {
        this.replyToUri = replyToUri;
    }
    
    public void registerCallback(Object messageId, WSIFAsyncResponseListener callback) {
        if(messageId == null) {
            throw new IllegalArgumentException();
        }
        synchronized(messageId2Callback) {
            messageId2Callback.put((URI)messageId, callback);
        }
    }
    
    public WSIFAsyncResponseListener unregisterCallback(Object messageId) {
        synchronized(messageId2Callback) {
            WSIFAsyncResponseListener callback = (WSIFAsyncResponseListener)
                messageId2Callback.remove((URI)messageId);
            return callback;
        }
        
    }
    
    public Object addCorrelationToOutgoingMessage(XmlDocument outgoingDoc)
        throws DynamicInfosetProcessorException
    {
        WsaMessageInformationHeaders  wsaHeaders = new WsaMessageInformationHeaders(outgoingDoc);
        WsaEndpointReference replyEpr;
        if(replyToEpr != null) {
            XmlElementAdapter aClone;
            try {
                aClone = ((XmlElementAdapter) replyToEpr.clone());
            } catch (CloneNotSupportedException e) {
                throw new DynamicInfosetProcessorException("could not clone replyTo element", e);
            }
            //WsaEndpointReference replyEpr = aClone.castOrWrap(aClone, WsaEndpointReference.class);
            replyEpr = (WsaEndpointReference) aClone;
        } else {
            replyEpr = new WsaEndpointReference(replyToUri);
        }
        wsaHeaders.setReplyTo(replyEpr);
        URI messageId = wsaHeaders.getMessageId();
        if(messageId == null) {
            messageId= URI.create("uuid:"+FastUUIDGen.nextUUID());
            wsaHeaders.setMessageId(messageId);
        }
        return messageId;
    }
}

/*
 * Indiana University Extreme! Lab Software License, Version 1.2
 *
 * Copyright (c) 2002-2004 The Trustees of Indiana University. All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 *
 * 2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 *
 * 3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 *
 * "This product includes software developed by the Indiana University Extreme!
 * Lab. For further information please visit http://www.extreme.indiana.edu/"
 *
 * Alternatively, this acknowledgment may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 *
 * 4) The name "Indiana University" or "Indiana University Extreme! Lab" shall
 * not be used to endorse or promote products derived from this software without
 * prior written permission from Indiana University. For written permission,
 * please contact http://www.extreme.indiana.edu/.
 *
 * 5) Products derived from this software may not use "Indiana University" name
 * nor may "Indiana University" appear in their name, without prior written
 * permission of the Indiana University.
 *
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of any
 * other entity. Indiana University disclaims any liability to any recipient for
 * claims brought by any other entity based on infringement of intellectual
 * property rights or otherwise.
 *
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 * INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 * GENERATED USING SOFTWARE.
 */



